<?php
/*
 * Clip Name: WP Clips v3 Clips Editor
 * Clip URI: https://clipbank.krolyn.com/clips/wp-clips-v3-clips-editor
 * Description: This Clip enables Clip editing via the WordPress Plugins Editor.
 * Version: 1.0
 * Author: Krolyn Studios
 * Author URI: https://clipbank.krolyn.com
 * Network: true
 *
 * @package WP_Clips/Precoded
 * @subpackage WP_Clips_v3_Clips_Editor
 */
 
 class WP_Clips_v3_Clips_Editor {


	public function __construct() {
		
		// Check compatible version
		if( version_compare( WP_CLIPS_VERS, '3.1', '<' ) ) {
			add_action( 'admin_print_footer_scripts', function() {
				printf( '<div class="notice notice-warning"><p>%s. <a href="https://wpclips.net/updating-wp-clips/" target="_blank">%s</a> %s.</p></div>',
					__( 'WP Clips v3 Clips Editor requires at least WP Clips v3.1', 'wp-clips' ),
					__( 'Click here', 'wp-clips' ),
					__( 'to update/upgrade', 'wp-clips' )
				);
			});
			rename( __DIR__, str_replace( '/precoded/', '/unclipped/', __DIR__ ) );
			return;
		}
		
		add_action( 'admin_init', [ $this, 'wpc_deprecation_check' ] );
	}
	
	// Start editor after checks
	public function wpc_deprecation_check() {
		
		if( class_exists( 'WP_Clips_v3_Custom_Clip_Editor' ) ) {
			add_action( 'admin_print_footer_scripts', function() {
				printf( '<div class="notice notice-warning"><p><b>%s</b> %s.</p></div>',
					__( 'WP Clips v3 Custom Clip Editor', 'wp-clips' ),
					__( 'is deprecated and must be deleted', 'wp-clips' )
				);
			});
			rename( __DIR__, str_replace( '/precoded/', '/unclipped/', __DIR__ ) );
			return;
		}

		add_filter( 'wp_clips_manager_clip_meta', [ $this, 'wpc_add_clip_edit_link' ], 10, 3 );
		$this->wpc_clips_editor();
	}

	// Clip editor (uses default WP plugin editor)
	public function wpc_clips_editor() {

		if( isset( $_REQUEST['file'] ) && strpos( $_REQUEST['file'], 'wp-clips/temp-' ) !== false ) {

			$file = explode( '/', $_REQUEST['file'] );
			$temp = explode( '-', $file[1], 2 );
			$source = $this->native_clip( $temp );
			$target = WP_CLIPS_ROOT . $file[1];

			if( ! file_exists( $source ) || file_exists( $target ) ) return;

			// Build temporary Clip folder and files
			mkdir( $target, 0755, true );

			foreach( $itr = new RecursiveIteratorIterator(
				new RecursiveDirectoryIterator( $source, RecursiveDirectoryIterator::SKIP_DOTS ),
				RecursiveIteratorIterator::SELF_FIRST ) as $item ) 
			{
				$copy = $target . '/' . $itr->getSubPathName();
				if( ! file_exists( $copy ) ) {
					if( $item->isDir() ) mkdir( $copy );
					else copy( $item, $copy );
				}
			}

			// Hide active plugin edit warning
			add_action( 'admin_print_styles', function() {
				?><style>.active-plugin-edit-warning { display:none; }</style><?php
			});

			// Disable auto page refresh
			wp_deregister_script( 'heartbeat' );
		}
		else {

			foreach( glob( WP_CLIPS_ROOT . 'temp-*' ) as $source ) {

				$file = explode( '/', $source );
				$temp = explode( '-', end( $file ), 2 );
				$target = $this->native_clip( $temp );

				// Copy temporary files to Clip
				foreach( $itr = new RecursiveIteratorIterator(
					new RecursiveDirectoryIterator( $source, RecursiveDirectoryIterator::SKIP_DOTS ),
					RecursiveIteratorIterator::SELF_FIRST ) as $copy ) 
				{
					$item = $target . '/' . $itr->getSubPathName();
					if( file_exists( $item ) )
						if( ! $copy->isDir() && filemtime( $copy ) > filemtime( $item ) )
							copy( $copy, $item );
				}

				// Delete temporary Clip folder
				$this->delete( $source );
			}
		}
	}

	// Add link to Clip meta in control
	public function wpc_add_clip_edit_link( $html, $slug, $data ) {

		$id = ! is_multisite() || is_network_admin() ? '' :  get_current_blog_id() . '-';
		
		if( ! $data['clipActv'] || ( ! empty( $id ) && basename( dirname( $data['clipPath'], 2 ) ) === 'network' ) )
			return $html;

		if( ! empty( $data['clipEdit'] ) || $slug == 'clip-custom' ) {

			$file = ( $slug !== 'clip-custom' ? 'clip' : 'custom' ) . '-functions.php';

			$url = sprintf( '%s%s?file=wp-clips/temp-%s%s/%s&plugin=wp-clips/wp-clips.php',
						is_multisite() ? network_admin_url() : admin_url(),
						'plugin-editor.php',
						$id,
						$data['clipFile'],
						$file
					);

			$html .= sprintf( ' &#10141; <a href="%s" onclick="return confirm(%s)">%s</a>',
						esc_url( $url ),
						__( "'IMPORTANT! Backups are recommended prior to making changes. Edits are applied only upon exiting WP Plugin Editor.'", 'wp-clips' ),
						__( 'Edit in WP Plugin Editor', 'wp-clips' ) );
		}
		return $html;
	}
	
	// Get origin Clip from temp filename
	private function native_clip( $temp ) {
		
		if( substr( $temp[1], 0, 5 ) !== 'clip-' ) {
			$temp = explode( '-', $temp[1], 2 );
			$id = $temp[0] . '-';
		}
				
		$path = empty( $id ) ? WP_CLIPS_DIR . ( is_multisite() ? 'network/' : '' ) : WP_CLIPS_SITE . rtrim( $id, '-' ) . '/';
		$native = $path . ( $temp[1] == 'clip-custom' ? '' : 'precoded/' ) . $temp[1];
		return $native;
	}

	// Directory deletions
	private function delete( $dir ) {

		foreach( new RecursiveIteratorIterator(
			new RecursiveDirectoryIterator( $dir, RecursiveDirectoryIterator::SKIP_DOTS ),
			RecursiveIteratorIterator::CHILD_FIRST ) as $del )
		{
			if( $del->isDir() )
				rmdir( $del->getPathname() );
			else
				unlink( $del->getPathname() );
		}
		rmdir( $dir );
	}
}

new WP_Clips_v3_Clips_Editor();